==> sagua_0.1.2-1.tar.gz: sagua-0.1.2/include/audio.h <==
#ifndef GUARD_audio_h
#define GUARD_audio_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file audio.h
/// @brief Class for libao wrapping.

#include <map>
#include <string>
#include <vector>

#include <cstdlib>
#include <cmath>

#include <ao/ao.h>

#include <glibmm.h>

#include "effect.h"

class MainManager;

/// @brief Class for libao wrapping
class Audio 
{
    public:
	~Audio();

	/// @brief Create Audio Object (singleton pattern).
	/// @param mainManager Program control MainManager
	/// @return Pointer of created Audio object.
	static std::auto_ptr<Audio> Create(MainManager& mainManager)
	{
	    if(audio_.get() == NULL)
		audio_ = std::auto_ptr<Audio>(new Audio(mainManager));

	    return audio_;
	}

	void SetDriverFromPreference();

	bool TestDriver(const char* const driverName); 
	bool TestDriver(const char* const driverName, const std::map<std::string, std::string>& option); 

	bool OpenDevice(const unsigned int& bits, const unsigned int& channels, const unsigned int& sampleRate, const unsigned int& byteFormat);

	/// @brief Close opened device.
	///
	/// Stop buffer play thread and close opened device.
	void Close()
	{
	    if(playLoopStart_) {
		playLoopStop_ = true;
		playLoopCondPush_.broadcast();
		playLoopThread_->join();
		playLoopStop_ = false;
		
		playLoopStart_ = false;	
	    }
	    
	    if(CheckOpenDevice() && bufferCount_ != 0 && !skipBufferFlush_) {
		
		ao_play(device_, buffer_, bufferCount_);
		
	    }

	    bufferCount_ = 0;
	    playLoopPause_ = false;
	    skipBufferFlush_ = false;

	    memset(buffer_, 0, 142848);
	    
	    ao_close(device_);
	    device_ = NULL;
	}

	/// @brief Check device is open.
	/// @return If device is opened, return true.
	bool CheckOpenDevice()
	{
	    if(device_)
		return true;
	    else
		return false;
	}

	/// @brief Check machine uses big-endian.
	/// @return If machine uses big-endian, return true.
	bool IsBigEndian()
	{
	    if(ao_is_big_endian() == 1)
		return true;
	    else
		return false;	
	}

	void Play(float** const samples, const unsigned int& bytes);
	void Play(const std::vector<double>& samples, const unsigned int& bytes);

	/// @brief Toggle pause state.
	///
	/// If you change player's state in pause, you should use
	/// skipPlayBufferedSample to true. Else you can hear buffered
	/// sample. 
	///
	/// In short, you should use skipPlayBufferedSample
	/// if you not just unpause player.
	/// @param skipPlayBufferedSample Skip play buffered sample.
	void PauseToggle(const bool& skipPlayBufferedSample)
	{
	    if(skipPlayBufferedSample)
		skipBufferFlush_ = true;
	    
	    if(playLoopPause_)
		playLoopPause_ = false;
	    else if(playLoopStart_)
		playLoopPause_ = true;	
	}

	/// @brief Check pause state.
	/// @return If buffer play is paused, return true.
	const bool& Paused()
	{
	    return playLoopPause_;
	}

    private:
	Audio(MainManager&);
	Audio(const Audio&);
	Audio& operator=(const Audio&);

	static std::auto_ptr<Audio> audio_;

	struct Dither {
	    double dBuffer_[9];
	    double dTState_;
	    unsigned int dPos_;
	};

	// mad decoder output size is 4608, ogg vorbis output size is variable,
	// ffmpeg decoder output size is 4096. So i take 4608 * 31, and will 
	// work find.
	char buffer_[142848];			///< Buffer for play.
	unsigned int bufferCount_;		///< Size of filled buffer.

	const double doubleInverseRandMax_;	///< Inverse of RAND_MAX.
	double ditherRand_;			///< 
	double ditherTemp_;
	double ditherXe_;
	double ditherResult_;

	MainManager& mainManager_;		///< Program control MainManager

	ao_device* device_;
	ao_option* option_;
	ao_sample_format format_;
	int driverId_;

	Glib::Thread* playLoopThread_;		///< Thread to play buffer.
	Glib::Mutex playLoopMutex_;		///< Mutex to protect buffer.
	Glib::Cond playLoopCondPush_;		///< Condition variable for push buffer.
	Glib::Cond playLoopCondPop_;		///< Condition variable for pop buffer.
	bool playLoopStart_;			///< For check play buffer is started. 
	bool playLoopStop_;			///< To stop play buffer thread.
	bool playLoopPause_;			///< Play buffer pause state. 
	bool skipBufferFlush_;			///< For skip play buffered samples.

	unsigned int singleSampleSize_;		///< Size of single sample. Equal to bits / 8 * channels.
	unsigned int bufferSize_;		///< Size of buffer for effector and dithering.
	bool littleEndian_;			///< For check endian of machine.

	short int tempScaledSample_;		///< Temporary variable for dithering result.

	Effect::Stereo stereoSignal_;		///< Temporary variable to apply effector.

	Dither dLeft_;				///< Dithering variable for left channel.
	Dither dRight_;				///< Dithering variable for right channel.

	void PlayBufferLoop();
	/// @brief Push sample buffer to play buffer
	///
	/// Push sample buffer to play buffer. Sample buffer size is
	/// member variable bufferSize_. 
	/// @param buf Sample buffer to push.
	void PlayBufferPush(const char* buf)
	{
	    Glib::Mutex::Lock lock(playLoopMutex_);

	    memcpy(buffer_ + bufferCount_, buf, bufferSize_);

	    bufferCount_ += bufferSize_;
	    
	    // If buffer filled over 4608 * 30, need to flush
	    while(bufferCount_ > 138240 && !playLoopStop_) 
		playLoopCondPop_.wait(playLoopMutex_);

	    playLoopCondPush_.signal();	

	}

	/// @brief Simple modulo function.
	///
	/// Ugly and not safe modulo function, but for just simple modulo
	/// process.
	/// @return x modulo n 
	unsigned int Mod(const unsigned int& x, const unsigned int& n)
	{
	    if(x > n)
		return x - n;
	    else
		return x;	
	}

	void ScaleTo16Bit(const double&, Dither&);
	
	/// @brief Convert short integer to char.
	///
	/// This function does not check char's address boundry.
	void ShortIntToChar(const short int& in, char* out)
	{
	    // becareful address boundry! 
	    if(littleEndian_){
		*out = in & 0xff;
		*(out+1) = in >> 8; 
	    } else {
		*out = in >> 8;
		*(out+1) = in & 0xff;
	    }
	}

	void Reset();

	/// @brief Class for management of ao_device.
	///
	/// Close ao_device in destructor.
	class DeviceManager
	{
	    public:
		DeviceManager(): device_(NULL) {}
		/// @brief Constructor for OpenLive at initially.
		/// @param id Driver id. 
		/// @param format Format to open.
		/// @param option Driver option.
		DeviceManager(const int& id, ao_sample_format* const format, ao_option* const option)
		: device_(NULL)
		{
		    device_ = ao_open_live(id, format, option);
		}
		~DeviceManager()
		{
		    ao_close(device_);
		}

		/// @brief Open live device.
		/// @param id Driver id.
		/// @param format Format to open.
		/// @param option Driver option.
		/// @return Success of open.
		const bool OpenLive(const int& id, ao_sample_format* const format, ao_option* const option)
		{
		    device_ = ao_open_live(id, format, option);

		    return device_ != NULL;
		}

		/// @brief Check current device is valid.
		/// @return If device is not NULL, return true.
		const bool Valid() const
		{
		    return device_ != NULL;
		}

	    private:
		DeviceManager& operator=(const DeviceManager&);
		DeviceManager(const DeviceManager&);
		
		ao_device* device_;	

	};

	/// @brief Class for management of ao_option.
	///
	/// Free ao_option in destructor.
	class OptionManager
	{
	    public:
		OptionManager(): option_(NULL) {}
		~OptionManager()
		{
		    ao_free_options(option_);
		}

		/// @brief Append option. 
		void Append(const std::string& key, const std::string& value)
		{
		    ao_append_option(&option_, key.c_str(), value.c_str());
		}

		/// @brief Append option.
		void Append(char* const key, char* const value)
		{
		    ao_append_option(&option_, key, value);
		}

		/// @brief Get ao_option.
		ao_option* Get()
		{
		    return option_;
		}

	    private:
		OptionManager& operator=(const OptionManager&);
		OptionManager(const OptionManager&);

		ao_option* option_;
	};

	/// @brief Function object to append ao_option from std::map
	struct AoOptionAppend: std::binary_function<std::pair<std::string, std::string>, OptionManager*, void>
	{
	    void operator()(const std::pair<std::string, std::string>& pair, OptionManager* const option) const
	    {
		option->Append(pair.first, pair.second);
	    }
	};
	
};

#endif

==> sagua_0.2.0-1.tar.gz: sagua-0.2.0/include/audio.h <==
#ifndef GUARD_audio_h
#define GUARD_audio_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

/// @file audio.h
/// @brief Class for libao wrapping.

#include <map>
#include <string>
#include <vector>

#include <cstdlib>
#include <cmath>

#include <ao/ao.h>

#include <glibmm.h>

#include "effect.h"

class MainManager;

/// @brief Class for libao wrapping
class Audio 
{
    public:
	~Audio();

	/// @brief Create Audio Object (singleton pattern).
	/// @param mainManager Program control MainManager
	/// @return Pointer of created Audio object.
	static std::auto_ptr<Audio> Create(MainManager& mainManager)
	{
	    if(audio_.get() == NULL)
		audio_ = std::auto_ptr<Audio>(new Audio(mainManager));

	    return audio_;
	}

	void SetDriverFromPreference();

	bool TestDriver(const char* const driverName); 
	bool TestDriver(const char* const driverName, const std::map<std::string, std::string>& option); 

	bool OpenDevice(const unsigned int& bits, const unsigned int& channels, const unsigned int& sampleRate, const unsigned int& byteFormat);

	/// @brief Close opened device.
	///
	/// Stop buffer play thread and close opened device.
	void Close(const bool& bufferFlush)
	{
	    if(playLoopStart_) {
		playLoopStop_ = true;
		playLoopCondPush_.broadcast();
		playLoopThread_->join();
		playLoopStop_ = false;
		
		playLoopStart_ = false;	
	    }
	    
	    if(CheckOpenDevice() && bufferCount_ != 0 && !skipBufferFlush_ && bufferFlush) {
		
		ao_play(device_, buffer_, bufferCount_);
		
	    }

	    bufferCount_ = 0;
	    playLoopPause_ = false;
	    skipBufferFlush_ = false;

	    memset(buffer_, 0, 142848);
	    
	    ao_close(device_);
	    device_ = NULL;
	}

	/// @brief Check device is open.
	/// @return If device is opened, return true.
	bool CheckOpenDevice()
	{
	    if(device_)
		return true;
	    else
		return false;
	}

	/// @brief Check machine uses big-endian.
	/// @return If machine uses big-endian, return true.
	bool IsBigEndian()
	{
	    if(ao_is_big_endian() == 1)
		return true;
	    else
		return false;	
	}

	void Play(float** const samples, const unsigned int& bytes);
	void Play(double* const samples, const unsigned int& bytes);
	void Play(const std::vector<double>& samples, const unsigned int& bytes);

	/// @brief Toggle pause state.
	///
	/// If you change player's state in pause, you should use
	/// skipPlayBufferedSample to true. Else you can hear buffered
	/// sample. 
	///
	/// In short, you should use skipPlayBufferedSample
	/// if you not just unpause player.
	/// @param skipPlayBufferedSample Skip play buffered sample.
	void PauseToggle(const bool& skipPlayBufferedSample)
	{
	    if(skipPlayBufferedSample)
		skipBufferFlush_ = true;
	    
	    if(playLoopPause_)
		playLoopPause_ = false;
	    else if(playLoopStart_)
		playLoopPause_ = true;	
	}

	/// @brief Check pause state.
	/// @return If buffer play is paused, return true.
	const bool& Paused()
	{
	    return playLoopPause_;
	}

    private:
	Audio(MainManager&);
	Audio(const Audio&);
	Audio& operator=(const Audio&);

	static std::auto_ptr<Audio> audio_;

	struct Dither {
	    double dBuffer_[9];
	    double dTState_;
	    unsigned int dPos_;
	};

	// mad decoder output size is 4608, ogg vorbis output size is variable,
	// ffmpeg decoder output size is 4096. So i take 4608 * 31, and will 
	// work find.
	char buffer_[142848];			///< Buffer for play.
	unsigned int bufferCount_;		///< Size of filled buffer.

	const double doubleInverseRandMax_;	///< Inverse of RAND_MAX.
	double ditherRand_;			///< 
	double ditherTemp_;
	double ditherXe_;
	double ditherResult_;

	MainManager& mainManager_;		///< Program control MainManager

	std::map<std::string, void (Audio::*)()> appendOptionCallbacks_; 

	ao_device* device_;
	ao_option* option_;
	ao_sample_format format_;
	ao_sample_format defaultFormat_;	///< Default sample format, 16 bits, 2 channels, 44.1 kHz, native byte-order.
	int driverId_;

	Glib::Thread* playLoopThread_;		///< Thread to play buffer.
	Glib::Mutex playLoopMutex_;		///< Mutex to protect buffer.
	Glib::Cond playLoopCondPush_;		///< Condition variable for push buffer.
	Glib::Cond playLoopCondPop_;		///< Condition variable for pop buffer.
	bool playLoopStart_;			///< For check play buffer is started. 
	bool playLoopStop_;			///< To stop play buffer thread.
	bool playLoopPause_;			///< Play buffer pause state. 
	bool skipBufferFlush_;			///< For skip play buffered samples.

	unsigned int singleSampleSize_;		///< Size of single sample. Equal to bits / 8 * channels.
	unsigned int bufferSize_;		///< Size of buffer for effector and dithering.
	bool littleEndian_;			///< For check endian of machine.

	short int tempScaledSample_;		///< Temporary variable for dithering result.

	Effect::Stereo stereoSignal_;		///< Temporary variable to apply effector.

	Dither dLeft_;				///< Dithering variable for left channel.
	Dither dRight_;				///< Dithering variable for right channel.

	void PlayBufferLoop();
	/// @brief Push sample buffer to play buffer
	///
	/// Push sample buffer to play buffer. Sample buffer size is
	/// member variable bufferSize_. 
	/// @param buf Sample buffer to push.
	void PlayBufferPush(const char* buf)
	{
	    Glib::Mutex::Lock lock(playLoopMutex_);

	    memcpy(buffer_ + bufferCount_, buf, bufferSize_);

	    bufferCount_ += bufferSize_;
	    
	    // If buffer filled over 4608 * 30, need to flush
	    while(bufferCount_ > 138240 && !playLoopStop_) 
		playLoopCondPop_.wait(playLoopMutex_);

	    playLoopCondPush_.signal();	

	}

	/// @brief Simple modulo function.
	///
	/// Ugly and not safe modulo function, but for just simple modulo
	/// process.
	/// @return x modulo n 
	unsigned int Mod(const unsigned int& x, const unsigned int& n)
	{
	    if(x > n)
		return x - n;
	    else
		return x;	
	}

	void ScaleTo16Bit(const double&, Dither&);
	
	/// @brief Convert short integer to char.
	///
	/// This function does not check char's address boundry.
	void ShortIntToChar(const short int& in, char* out)
	{
	    // becareful address boundry! 
	    if(littleEndian_){
		*out = in & 0xff;
		*(out+1) = in >> 8; 
	    } else {
		*out = in >> 8;
		*(out+1) = in & 0xff;
	    }
	}

	void Reset();

	/// @brief Class for management of ao_device.
	///
	/// Close ao_device in destructor.
	class DeviceManager
	{
	    public:
		DeviceManager(): device_(NULL) {}
		/// @brief Constructor for OpenLive at initially.
		/// @param id Driver id. 
		/// @param format Format to open.
		/// @param option Driver option.
		DeviceManager(const int& id, ao_sample_format* const format, ao_option* const option)
		: device_(NULL)
		{
		    device_ = ao_open_live(id, format, option);
		}
		~DeviceManager()
		{
		    ao_close(device_);
		}

		/// @brief Open live device.
		/// @param id Driver id.
		/// @param format Format to open.
		/// @param option Driver option.
		/// @return Success of open.
		const bool OpenLive(const int& id, ao_sample_format* const format, ao_option* const option)
		{
		    device_ = ao_open_live(id, format, option);

		    return device_ != NULL;
		}

		/// @brief Check current device is valid.
		/// @return If device is not NULL, return true.
		const bool Valid() const
		{
		    return device_ != NULL;
		}

	    private:
		DeviceManager& operator=(const DeviceManager&);
		DeviceManager(const DeviceManager&);
		
		ao_device* device_;	

	};

	/// @brief Class for management of ao_option.
	///
	/// Free ao_option in destructor.
	class OptionManager
	{
	    public:
		OptionManager(): option_(NULL) {}
		~OptionManager()
		{
		    ao_free_options(option_);
		}

		/// @brief Append option. 
		void Append(const std::string& key, const std::string& value)
		{
		    ao_append_option(&option_, key.c_str(), value.c_str());
		}

		/// @brief Append option.
		void Append(char* const key, char* const value)
		{
		    ao_append_option(&option_, key, value);
		}

		/// @brief Get ao_option.
		ao_option* Get()
		{
		    return option_;
		}

	    private:
		OptionManager& operator=(const OptionManager&);
		OptionManager(const OptionManager&);

		ao_option* option_;
	};

	/// @brief Function object to append ao_option from std::map
	struct AoOptionAppend: std::binary_function<std::pair<std::string, std::string>, OptionManager*, void>
	{
	    void operator()(const std::pair<std::string, std::string>& pair, OptionManager* const option) const
	    {
		option->Append(pair.first, pair.second);
	    }
	};

	void ApplyEffectorAndDither(const double& left, const double& right, char* output);

	/// @brief Find driver name in appendOptionCallbacks_, call it.
	void AppendOptionFromPreference(const std::string& driverName)
	{
	    std::map<std::string, void (Audio::*)()>::const_iterator iter = appendOptionCallbacks_.find(driverName);

	    if(iter != appendOptionCallbacks_.end())
		(this->*(iter->second))();
	}

	void AppendAlsaOptionFromPreference();
	void AppendAlsa9OptionFromPreference();
	void AppendOSSOptionFromPreference();
	void AppendSunOptionFromPreference();
	void AppendAixsOptionFromPreference();
	
};

#endif
